from setuptools import setup, find_packages

setup(
    name='Music-Player',
    version='1.0.5.1',
    packages=['MusicPlayer'],
    install_requires=[
        'PyQt5',
        'pycryptodome',
        'eyed3==0.8.0',
        'quamash',
        'requests'
    ],
    package_data={
        '': ['apis/*.py',
        'features/*.py',
        'networks/*.py',
        'QSS/*.qss',
        'resource/*.png',
        'resource/*.ico',
        'widgets/*.py',
        'logger/*.py',
        'dbManager/*.py'],
        },
    entry_points={
        'console_scripts': [
            'musicplayer = MusicPlayer.music:start'
        ],
    },

    license='MIT',
    author='cyrbuzz',
    author_email='cyrbuzz@foxmail.com',
    url='https://github.com/HuberTRoy/MusicPlayer',
    description='A beautiful music player for everyone. Enjoy yourself~.',
    keywords=['music', 'netease', 'xiami', 'qq', 'player'],
)


__author__ = 'cyrbuzz'

from singsFrameBase import SingsFrameBase, SingsSearchResultFrameBase


class XiamiSingsArea(SingsFrameBase):

    def __init__(self, parent=None):
        super(XiamiSingsArea, self).__init__(parent)


class XiamiSearchResultFrame(SingsSearchResultFrameBase):

    def __init__(self, parent=None):
        super(XiamiSearchResultFrame, self).__init__(parent)
        


__author__ = 'cyrbuzz'
"""不要单独运行。"""
from singsFrameBase import SingsFrameBase, SingsSearchResultFrameBase


# 一个Tab，网易云的全部歌单。
class NetEaseSingsArea(SingsFrameBase):
    """全部歌单。"""

    def __init__(self, parent=None):
        super(NetEaseSingsArea, self).__init__(parent)


class NetEaseSearchResultFrame(SingsSearchResultFrameBase):

    def __init__(self, parent):
        super(NetEaseSearchResultFrame, self).__init__(parent)


# -*- coding: utf-8 -*-

# This file copy from https://github.com/wn0112/PPlayer

from PyQt5 import QtWidgets, QtGui, QtCore

def _fromUtf8(s):
    return s

class Button(QtWidgets.QPushButton):
    def __init__(self,parent = None):
        super(Button,self).__init__(parent)
        self.status = 0 

    def loadPixmap(self, pic_name):
        self.pixmap = QtGui.QPixmap(pic_name)
        self.btn_width = self.pixmap.width()
        self.btn_height = self.pixmap.height()/6
        self.setFixedSize(self.btn_width, self.btn_height)

    def mousePressEvent(self,event):
        if event.button() == QtCore.Qt.LeftButton:
            self.status = 2
            self.update()
            self.clicked.emit(True)

    def mouseReleaseEvent(self,event):
        if event.button() == QtCore.Qt.LeftButton:
            self.status = 0 
            self.update()
            self.released.emit()

    def rst(self):
        self.status = 0
        self.update()

    def rbReleased(self):
        self.status = 0
        self.update()

    def rbPressed(self):
        self.status = 4
        self.update()

    def lbPressed(self):
        self.status = 3
        self.update()

    def lbReleased(self):
        self.status = 0
        self.update()

    def bothPressed(self):
        self.status = 5
        self.update()
        
    def bothReleased(self):
        self.status = 0
        self.update()

    def paintEvent(self,event):
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        self.painter.drawPixmap(self.rect(), self.pixmap.copy(0, self.btn_height * self.status, self.btn_width, self.btn_height))
        self.painter.end()

class PlayButton(Button):
    def __init__(self,parent = None):
        super(PlayButton,self).__init__(parent)
        self.setCheckable(True)

    def mouseReleaseEvent(self, event):
        self.released.emit()

class SRButton(PlayButton):
    def __init__(self,parent = None):
        super(SRButton,self).__init__(parent)
        self.syn = 0

    def mousePressEvent(self,event):    
        if event.button() == QtCore.Qt.LeftButton:
            self.status = 2 
            self.update()
            if not self.isChecked():
                self.syn = 1
                self.clicked.emit(True)        

    def mouseReleaseEvent(self, event):
        if self.isChecked() and self.syn != 1:
            self.clicked.emit(True)
            self.released.emit()
        else:
            self.syn = 0


class PushButton(QtWidgets.QPushButton):
    def __init__(self,parent = None):
        super(PushButton,self).__init__(parent)
        self.status = 0 

    def loadPixmap(self, pic_name):
        self.pixmap = QtGui.QPixmap(pic_name)
        self.btn_width = self.pixmap.width()/4
        self.btn_height = self.pixmap.height()
        self.setFixedSize(self.btn_width, self.btn_height)

    def enterEvent(self,event):
        if not self.isChecked() and self.isEnabled():
            self.status = 1 
            self.update()

    def setDisabled(self, bool):
        super(PushButton,self).setDisabled(bool)
        if not self.isEnabled():
            self.status = 2
            self.update()
        else:
            self.status = 0
            self.update()

    def mousePressEvent(self,event):
        if event.button() == QtCore.Qt.LeftButton:
            self.status = 2 
            self.update()    

    def mouseReleaseEvent(self,event):
        if event.button() == QtCore.Qt.LeftButton: 
            self.clicked.emit(True)
        if not self.isChecked():
            self.status = 3
        if self.menu():
            self.menu().exec_(event.globalPos())
        self.update()
        
    def leaveEvent(self,event):
        if not self.isChecked() and self.isEnabled():
            self.status = 0 
            self.update()

    def paintEvent(self,event):
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        self.painter.drawPixmap(self.rect(), self.pixmap.copy(self.btn_width * self.status, 0, self.btn_width, self.btn_height))
        self.painter.end()
        
class PushButton2(QtWidgets.QPushButton):
    def __init__(self,parent = None):
        super(PushButton2,self).__init__(parent)

    def loadPixmap(self, pic_name):
        self.pixmap = QtGui.QPixmap(pic_name)
        self.btn_width = self.pixmap.width()
        self.btn_height = self.pixmap.height()
        self.setFixedSize(self.btn_width, self.btn_height)

    def paintEvent(self,event):
        self.painter = QtGui.QPainter()
        self.painter.begin(self)
        self.painter.drawPixmap(self.rect(), self.pixmap.copy(0, 0, self.btn_width, self.btn_height))
        self.painter.end()    


__author__ = 'cyrbuzz'

import os
import re
import glob
import pickle
import os.path
import logging
try:
    import eyed3
except ImportError:
    print('eyed3没有成功加载或安装，当再次打开时下载的音乐会加载不到！')

from apiRequestsBase import HttpRequest
from asyncBase import aAsync, toTask
from base import QFileDialog, QObject, QTableWidgetItem, checkFolder
from addition import itv2time

import logger

logger = logging.getLogger(__name__)

myRequests = HttpRequest()


def getAllFolder(topFolder):
    result = []

    def findFolder(topFolder):
        folders = [os.path.join(topFolder, i) for i in os.listdir(topFolder) if not os.path.isfile(os.path.join(topFolder, i))]
        if not folders:
            return 
        else:
            result.extend(folders)
            for i in folders:
                findFolder(i)

    findFolder(topFolder)

    return result


def replace_forbidden_sym(string):

    return re.sub(r'[\\/:*?"<>|]{1}', ' ', string)


class ConfigDownloadFrame(QObject):
    myDownloadFrameCookiesFolder = 'cookies/downloadInfo/downloadFolder.cks'
    allCookiesFolder = [myDownloadFrameCookiesFolder]

    def __init__(self, downloadFrame):
        super(ConfigDownloadFrame, self).__init__()
        self.downloadFrame = downloadFrame
        self.showTable = self.downloadFrame.singsTable

        self.musicList = []
        self.folder = []
        self.myDownloadFolder = os.path.join(os.getcwd(), 'downloads')

        self._setDownloadFolder(self.myDownloadFolder)

        self.bindConnect()

        self.loadCookies()

    def bindConnect(self):
        self.downloadFrame.selectButton.clicked.connect(self.selectFolder)
        self.downloadFrame.singsTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)

    def getDownloadSignal(self):
        #
        window = self.downloadFrame.parent
        try:
            window.searchArea.config.download.connect(self.downloadSong)
            window.detailSings.config.download.connect(self.downloadSong)
        except Exception as e:
            logger.error("下载时遇到未知错误", exc_info=True)


    def _setDownloadFolder(self, folderName):
        logger.info("下载目标变更{}".format(folderName))
        self.fromPathLoadSong(folderName)
        self.myDownloadFolder = folderName
        self.downloadFrame.currentStorageFolder.setText(folderName)

    @toTask
    def downloadSong(self, musicInfo):
        logger.info("正在下载的音乐的信息: {}".format(musicInfo))

        url = musicInfo.get('url')
        allMusicName = re.search(r'.*\.[a-zA-Z0-9]+', url[url.rfind('/')+1:]).group(0)
        if allMusicName:

            musicSuffix = allMusicName[allMusicName.rfind('.')+1:]
            musicName = '{name}.{suf}'.format(name=musicInfo.get('name') + ' - ' + musicInfo.get('author'), suf=musicSuffix)
        else:
            # TODO MD5。
            musicName = "random_name.mp3"

        musicName = replace_forbidden_sym(musicName)

        self.downloadFrame.parent.systemTray.showMessage("~~~", '{musicName} 加入下载队列'.format(musicName=musicName))
        # TODO
        # Streaming.
        future = aAsync(myRequests.httpRequest, url, 'GET')
        data = yield from future

        localPath = '{myDownloadFolder}/{musicName}'.format(myDownloadFolder=self.myDownloadFolder, musicName=musicName)
        with open(localPath, 'wb') as f:
            f.write(data.content)

        musicInfo['url'] = localPath

        # 从托盘栏给出提示。
        self.downloadFrame.parent.systemTray.showMessage("~~~", '{musicName} 下载完成'.format(musicName=musicName))
        # 将音乐信息加到musicList中。
        self.musicList.append(musicInfo)
        self.updateDownloadShowTable(musicInfo)

    def updateDownloadShowTable(self, musicInfo):
        showInfo = [musicInfo.get("name"), musicInfo.get("author"), musicInfo.get("time")]
        # 这里写"我的下载"的实例对象。
        # 首先获取出当前总共多少行。
        rowCount = self.showTable.rowCount()
        self.showTable.setRowCount(rowCount+1)
        #  然后直接添加过去就好啦。
        for i in range(3):
            self.showTable.setItem(rowCount, i, QTableWidgetItem(showInfo[i]))

    def fromPathLoadSong(self, selectFolder):
        if not os.path.isdir(selectFolder):
            os.mkdir(selectFolder)
            return 
        mediaFiles = glob.glob(selectFolder+'/*.mp3')
        allFolder = getAllFolder(selectFolder)
        for i in allFolder:
            mediaFiles.extend(glob.glob(i+'/*.mp3'))

        length = len(mediaFiles)
        
        self.downloadFrame.singsTable.clearContents()
        self.downloadFrame.singsTable.setRowCount(length)
        self.musicList = []
        
        for i in enumerate(mediaFiles):
            music = eyed3.load(i[1])

            if not music:
                self.singsTable.removeRow(i[0])
                continue

            try:
                name = music.tag.title
                author = music.tag.artist

                if not name:
                    filePath = i[1].replace(selectFolder, '')
                    name = filePath[1:][:-4]
                if not author:
                    author = ''  
            except:
                try:
                    # TODO
                    # if more folders exist.
                    filePath = i[1].replace(selectFolder, '')
                    name = filePath[1:][:-4]
                except Exception as e:
                    name = i[1]
                    author = ''
            try:
                time = itv2time(music.info.time_secs)
            except:
                time = '00:00'

            self.musicList.append({'name': name, 'author': author, 'time': time, 'url': i[1], 'music_img': 'None'})
            self.downloadFrame.singsTable.setItem(i[0], 0, QTableWidgetItem(name))
            self.downloadFrame.singsTable.setItem(i[0], 1, QTableWidgetItem(author))
            self.downloadFrame.singsTable.setItem(i[0], 2, QTableWidgetItem(time))

    def selectFolder(self):
        folder = QFileDialog()
        selectFolder = folder.getExistingDirectory()
        if not selectFolder:
            pass
        else:
            self.folder.append(selectFolder)
            self._setDownloadFolder(selectFolder)
            self.fromPathLoadSong(selectFolder)

    @checkFolder(allCookiesFolder)
    def saveCookies(self):
        with open(self.myDownloadFrameCookiesFolder, 'wb') as f:
            pickle.dump(self.myDownloadFolder, f)

    @checkFolder(allCookiesFolder)
    def loadCookies(self):
        with open(self.myDownloadFrameCookiesFolder, 'rb') as f:
            self.myDownloadFolder = pickle.load(f)

        self._setDownloadFolder(self.myDownloadFolder)

    # 事件。
    def itemDoubleClickedEvent(self):
        currentRow = self.downloadFrame.singsTable.currentRow()
        data = self.musicList[currentRow]

        self.downloadFrame.parent.playWidgets.setPlayerAndPlayList(data)


import sqlite3


class DbManager(object):

    def __init__(self, *args):
        self.db = sqlite3.connect(*args)
        self.cursor = self.db.cursor()

    def __enter__(self):

        return self.cursor

    def __exit__(self, types, value, traceback):
        self.db.commit()

        return False

    def __del__(self):
    	self.db.commit()
    	self.db.close()

    def switchDb(self, *args):
        self.db.close()

        self.db = sqlite3.connect(*args)
        self.cursor = self.db.cursor()

    def createTable(self, tableString):
        self.cursor.execute(tableString)
        self.db.commit()

    def commitAndClose(self):
    	self.db.commit()
    	self.db.close()


__author__ = 'cyrbuzz'

import re
import os
import random

import asyncio
import pickle

import addition

from apiRequestsBase import HttpRequest
from asyncBase import aAsync, toTask
from base import QAction, QMenu, checkFolder, QIcon, QLabel, QObject, RequestThread, QTableWidgetItem, QCursor, pyqtSignal
from singsFrameBase import PlaylistButton
from netEaseApi import netease
from xiamiApi import xiami
from qqApi import qqApi
from userInfoManager import UserDbManager


transTime = addition.itv2time

myRequests = HttpRequest()


class ConfigWindow(QObject):

    def __init__(self, window):
        super(ConfigWindow, self).__init__()
        self.window = window
        
        self.window.db = UserDbManager()

        # 用于存储Tab的历史，方便前后切换。
        # 只存储5个，不考虑效率问题。
        self.history = []
        self.currentIndex = -1
        # 前后切换时也会触发currentChanged信号，
        # 前后切换时不允许增加新的历史也不允许删除旧的历史。
        self.isTab = False

        self.bindConnect()

    def addTab(self, widget, name=''):
        self.window.mainContents.addTab(widget, name)

    def allTab(self):
        return self.window.mainContents.count()

    def addTabHistory(self, index):
        length = len(self.history)
        if not self.isTab:
            if length < 5:
                self.history.append(index)
            else:
                self.history.pop(0)
                self.history.append(index)
            # 不是前后切换时将当前的索引定为末尾一个。
            self.currentIndex = length
        else:
            self.isTab = False

    def bindConnect(self):
        self.window.mainContents.currentChanged.connect(self.addTabHistory)

    def nextTab(self):
        # 后一个的切换。

        if self.currentIndex  == len(self.history)-1 or self.currentIndex == -1:
            return
        else:
            self.isTab = True
            self.currentIndex += 1
            self.window.mainContents.setCurrentIndex(self.history[self.currentIndex])

    def prevTab(self):
        # 前一个的切换。
        if self.currentIndex == 0 or self.currentIndex == -1:
            return
        else:
            self.isTab = True
            self.currentIndex -= 1
            self.window.mainContents.setCurrentIndex(self.history[self.currentIndex])

    def setTabIndex(self, index):
        self.window.mainContents.setCurrentIndex(index)

    def getDownloadFolder(self):
        return self.window.downloadFrame.config.myDownloadFolder

    def pullRecommendSong(self):
        # 1. 获取出前x个播放次数最多的歌曲。无则不变直接返回
        # 2. 随机挑选一个。 
        # 3. 获取包含它的歌单。 如果全部获取失败则重新进行2。
        # 4. 从歌单中挑选曲子添加到recommendFrame的musicList里。

        topSongInHistory = self.window.db.getSongByPlayTimes()

        if not topSongInHistory:
            return

        randList = list(range(len(topSongInHistory)))

        while 1:
            try:
                randSong = randList.pop(randList.index(random.choice(randList)))
            except IndexError:
                return

            songId = topSongInHistory[randSong]
            relativePlaylistIds = netease.getContainedPlaylists(songId)

            if not relativePlaylistIds:
                continue

            for i in relativePlaylistIds:
                songInfo = netease.getRandomSongFromPlaylist(i)
                if not songInfo:
                    continue
                # add to recommend and return
                self.window.recommendFrame.config.setSongs(songInfo)
                return


class ConfigHeader(QObject):
    loginCookiesFolder = 'cookies/headers/loginInfor.cks'
    allCookiesFolder = [loginCookiesFolder]

    def __init__(self, header):
        super(ConfigHeader, self).__init__()
        self.header = header


        self.loginThread = RequestThread(self, None, self.loginFinished)
        self.loginThread.breakSignal.connect(self.emitWarning)

        self.header.loginBox.connectLogin(self.login)
        
        self.loginInfor = {}
        self.result = None
        self.songsDetail = None
        # 用于确定登陆状态。
        self.code = 200
        # 用于确定是否最大化.   
        self.isMax = False

        self.bindConnect()
        self.loadCookies()

    def bindConnect(self):
        self.header.closeButton.clicked.connect(self.header.parent.close)
        self.header.showminButton.clicked.connect(self.header.parent.showMinimized)
        self.header.showmaxButton.clicked.connect(self.showMaxiOrRevert)
        self.header.loginButton.clicked.connect(self.showLoginBox)
        self.header.prevButton.clicked.connect(self.header.parent.config.prevTab)
        self.header.nextButton.clicked.connect(self.header.parent.config.nextTab)
        self.header.searchLine.setButtonSlot(lambda: self.search())

    def showMaxiOrRevert(self):
        if self.isMax:
            self.header.parent.showNormal()
            self.isMax = False
        else:
            self.header.parent.showMaximized()
            self.isMax = True

    @toTask
    def search(self):
        text = self.header.searchLine.text()
        future = aAsync(netease.search, text)
        self.result = yield from future
        if not self.result['songCount']:
            songsIds = []
            self.result['songs'] = []
        else: 
            songsIds = [i['id'] for i in self.result['songs']]

            self.songsDetail = {j:'http{0}'.format(i) for i, j in enumerate(songsIds)}
           
            # 进行重新编辑方便索引。
            songs = self.result['songs']
            self.result['songs'] = [{'name':i['name'], 
            'artists': i['ar'], 
            'picUrl': i['al']['picUrl'],
            'mp3Url': self.songsDetail[i['id']],
            'duration': i['dt'],
            'music_id':i['id']} for i in songs]

        songsCount = self.result['songCount']

        # 总数是0即没有找到。
        if not songsCount:
            songs = []
        else:
            songs = self.result['songs'] 

        self.header.parent.searchArea.setText(text)

        self.header.parent.searchArea.config.setSingsData(songs)

        self.header.parent.config.setTabIndex(5)

    def showLoginBox(self):
        self.header.loginBox.open()

    def login(self):
        informations = self.header.loginBox.checkAndGetLoginInformation()
        
        if not informations:
            return 

        self.loginThread.setTarget(self.loadLoginInformations)
        self.loginThread.setArgs(informations)
        self.loginThread.start()

    def loadLoginInformations(self, informations:tuple):
        result = netease.login(*informations)
        # 网络不通或其他问题。
        if not result:
            self.loginThread.breakSignal.emit('请检查网络后重试~.')
            self.code = 500
            return

        code = result.get('code')
        if str(code) != '200':
            self.loginThread.breakSignal.emit(str(result.get('msg')))
            self.code = 500
            return 

        self.loginInfor = result
        self.code = 200

    def loginFinished(self):
        if str(self.code) == '200': 
            self.header.loginBox.accept()
            self.setUserData()

    @toTask
    def setUserData(self):
        profile = self.loginInfor['profile']
        avatarUrl = profile['avatarUrl']
        self.header.userPix.setSrc(avatarUrl)
        # 加载该账户创建及喜欢的歌单。
        userId = profile['userId']
        future = aAsync(netease.user_playlist, userId)
        data = yield from future

        nickname = profile['nickname']
        self.header.loginButton.setText(nickname)

        self.header.loginButton.clicked.disconnect()
        self.header.loginButton.clicked.connect(self.exitLogin)
        
        self.header.parent.navigation.config.setPlaylists(data)
  
    def emitWarning(self, warningStr):
        self.header.loginBox.setWarningAndShowIt(warningStr)

    def exitLogin(self):
        self.loginInfor = {}
        self.header.loginButton.setText('未登录 ▼')
        self.header.loginButton.clicked.connect(self.showLoginBox)
        self.header.userPix.setSrc('resource/nouser.png')
        self.header.parent.navigation.config.clearPlaylists()

    @checkFolder(allCookiesFolder)
    def saveCookies(self):
        with open(self.loginCookiesFolder, 'wb') as f:
            pickle.dump(self.loginInfor, f)

    @checkFolder(allCookiesFolder)
    def loadCookies(self):
        with open(self.loginCookiesFolder, 'rb') as f:
            self.loginInfor = pickle.load(f)
        self.setUserData()
        self.header.loginButton.clicked.disconnect()
        self.header.loginButton.clicked.connect(self.exitLogin)


class ConfigNavigation(QObject):

    def __init__(self, navigation):
        super(ConfigNavigation, self).__init__()
        self.navigation = navigation
        
        self.detailFrame = self.navigation.parent.detailSings
        # window
        self.mainContents = self.navigation.parent

        self.nativeListFunction = self.tabNativeFrame
        self.singsFunction = self.none
        
        self.playlists = []

        self.result = None
        self.singsUrls = None
        self.coverImgUrl = None

        self.api = netease

        self.bindConnect()

    def bindConnect(self):
        self.navigation.navigationList.itemPressed.connect(self.navigationListItemClickEvent)
        self.navigation.nativeList.itemPressed.connect(self.nativeListItemClickEvent)
    
    def navigationListItemClickEvent(self):
        """用户处理导航栏的点击事件。"""
        # 处理其他组件取消选中。
        for i in self.playlists:
            if i.isChecked():
                i.setCheckable(False)
                i.setCheckable(True)
                break

        self.navigation.nativeList.setCurrentRow(-1)

        """处理事件。"""
        self.navigationListFunction()

    def nativeListItemClickEvent(self, item):
        """本地功能的点击事件。"""
        for i in self.playlists:
            if i.isChecked():
                i.setCheckable(False)
                i.setCheckable(True)
                break

        self.navigation.navigationList.setCurrentRow(-1)

        """处理事件。"""
        self.nativeListFunction(item)

    def singsButtonClickEvent(self):
        """歌单的点击事件。"""
        self.navigation.navigationList.setCurrentRow(-1)
        self.navigation.nativeList.setCurrentRow(-1)

        """处理事件。"""
        self.singsFunction()

    def setPlaylists(self, datas):
        # 布局原因，需要在最后加一个stretch才可以正常布局。
        # 所以这边先将最后一个stretch删去，将所有的内容添加完成后在加上。
        self.navigation.mainLayout.takeAt(self.navigation.mainLayout.count()-1)
        for i in datas:
            button = PlaylistButton(self, i['id'], i['coverImgUrl'], QIcon('resource/notes2.png'), i['name'])
            button.hasClicked.connect(self.startRequest)

            self.playlists.append(button)
            self.navigation.mainLayout.addWidget(button)       
        
        self.navigation.mainLayout.addStretch(1)

    def clearPlaylists(self):
        for i in self.playlists:
            i.deleteLater()

        self.playlists = []

        for i in range(11, self.navigation.mainLayout.count()):
            self.navigation.mainLayout.takeAt(i)

        self.navigation.mainLayout.addStretch(1)

    @toTask
    def startRequest(self, ids, coverImgUrl):
        self.coverImgUrl = coverImgUrl
        self.singsButtonClickEvent()

        future = aAsync(self.api.details_playlist, ids)
        self.result = yield from future
        # 由于旧API不在直接返回歌曲地址，需要获取歌曲号后再次进行请求。
        singsIds = [i['id'] for i in self.result['tracks']]
        # 此处还有些问题。
        # 由于是两次url请求，稍微变得有点慢。
        # future = aAsync(self.api.singsUrl, singsIds)
        # data = yield from future
        # self.singsUrls = {i['id']:i['url'] for i in data}
        # self.singsUrls = [self.singsUrls[i] for i in singsIds]
        self.singsUrls = ['http{0}'.format(i) for i, j in enumerate(singsIds)]

        self.detailFrame.config.setupDetailFrames(self.result, self.singsUrls, singsIds)
        self.detailFrame.picLabel.setSrc(self.coverImgUrl)
        self.detailFrame.picLabel.setStyleSheet('''QLabel {padding: 10px;}''')

        # 隐藏原来的区域，显示现在的区域。
        self.mainContents.mainContents.setCurrentIndex(1)

    def navigationListFunction(self):
        isVisible = self.navigation.parent.mainContent.tab.isVisible()
        if self.navigation.navigationList.currentRow() == 0:
            # 发现音乐。
            self.navigation.parent.mainContents.setCurrentIndex(0)

    def tabNativeFrame(self, item):
        if item.text() == ' 本地音乐':
            self.mainContents.mainContents.setCurrentIndex(2)
        elif item.text() == ' 我的下载':
            self.mainContents.mainContents.setCurrentIndex(3)
        elif item.text() == ' 专属推荐':
            self.mainContents.mainContents.setCurrentIndex(4)

    def none(self):
        pass


class ConfigMainContent(QObject):

    def __init__(self, mainContent):
        super(ConfigMainContent, self).__init__()
        self.mainContent = mainContent


class ConfigSearchArea(QObject):

    download = pyqtSignal(dict)
    def __init__(self, searchArea):
        super(ConfigSearchArea, self).__init__()

        # current show-table's index.
        self.currentIndex = 0
        # current widgets name。
        self.currentName = '网易云'

        # parent.
        self.searchArea = searchArea
        
        # get storage folder
        self.downloadFolder = self.searchArea.parent.config.getDownloadFolder()

        self.transTime = addition.itv2time
        
        self.searchEngineers = {'网易云': netease, '虾米': xiami, 'QQ': qqApi}
        # TODO 
        # to config singsFrameBase instead of configing them respective.
        self.searchResultTableIndexs = {'网易云':self.searchArea.neteaseSearchFrame.singsResultTable, 
            '虾米':self.searchArea.xiamiSearchFrame.singsResultTable , 
            'QQ':self.searchArea.qqSearchFrame.singsResultTable}

        self.musicList = []
        self.noContents = "很抱歉 未能找到关于<font style='text-align: center;' color='#23518F'>“{0}”</font>的{1}。"

        self.bindConnect()
        self.setContextMenu()

    def bindConnect(self):
        self.searchArea.contentsTab.tabBarClicked.connect(self.searchBy)
        self.searchArea.neteaseSearchFrame.singsResultTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)
        self.searchArea.xiamiSearchFrame.singsResultTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)
        self.searchArea.qqSearchFrame.singsResultTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)
        
        self.searchArea.neteaseSearchFrame.singsResultTable.contextMenuEvent = self.contextEvent
        self.searchArea.xiamiSearchFrame.singsResultTable.contextMenuEvent = self.contextEvent
        self.searchArea.qqSearchFrame.singsResultTable.contextMenuEvent = self.contextEvent

    def setContextMenu(self):
        self.actionDownloadSong = QAction('下载', self)
        self.actionDownloadSong.triggered.connect(self.downloadSong)

    @toTask
    def downloadSong(self, x):
        # x is useless, but must be.
        # 
        musicInfo = self.musicList[self.currentIndex]
        url = musicInfo.get('url')
        if 'http:' not in url and 'https:' not in url:
                songId = musicInfo.get('music_id')
                future = aAsync(netease.singsUrl, [songId])
                url = yield from future
                url = url[0].get('url')
                musicInfo['url'] = url

        self.download.emit(musicInfo)

    def searchBy(self, index):
        currentWidgetName = self.searchArea.contentsTab.tabText(index)
        self.currentName = currentWidgetName
        self.search(currentWidgetName)

    @toTask
    def search(self, name):
        """接受name信息，由这个引擎进行搜索。"""
        searchEngineer = self.searchEngineers[name]
        data = yield from aAsync(searchEngineer.search, self.searchArea.text)
        if not data['songCount']:
            songsIds = []
            data['songs'] = []
        else: 
            songsIds = [i['id'] for i in data['songs']]

            if name == '网易云':
                songsDetail = {i:'http' for i in songsIds}
            elif name == '虾米' or name == 'QQ':
                songsDetail = {i['id']:i['mp3Url'] for i in data['songs']}

            # 进行重新编辑方便索引。
            songs = data['songs']
            data['songs'] = [{'name':i['name'], 
            'artists': i['ar'], 
            'picUrl': i['al']['picUrl'],
            'mp3Url': songsDetail[i['id']],
            'duration': i['dt'],
            'music_id':i['id'],
            'lyric': i.get('lyric')} for i in songs]

        songsCount = data['songCount']

        # 总数是0即没有找到。
        if not songsCount:
            songs = []
        else:
            songs = data['songs'] 

        self.setSingsData(songs)

    def setSingsData(self, data):
        # 单曲搜索结果。
        searchArea = self.searchArea.contentsTab.currentWidget()
        if not len(data):
            # self.contentsTab.addTab()
            searchArea.noSingsContentsLabel.setText(self.noContents.format(self.searchArea.text, '单曲'))
            searchArea.singsResultTable.hide()
            searchArea.noSingsContentsLabel.show()
        else:
            searchArea.singsResultTable.setRowCount(len(data))

            musicList = []
            for count, datas in enumerate(data):
                picUrl = datas['picUrl']
                url = datas['mp3Url']
                name = datas['name']
                authors = ','.join([t['name'] for t in datas['artists']])
                duration = self.transTime(datas['duration']/1000)
                musicId = datas['music_id']

                searchArea.singsResultTable.setItem(count, 0, QTableWidgetItem(name))
                searchArea.singsResultTable.setItem(count, 1, QTableWidgetItem(authors))
                searchArea.singsResultTable.setItem(count, 2, QTableWidgetItem(duration))
                musicList.append({'url': url, 
                    'name': name, 
                    'time':duration, 
                    'author':authors, 
                    'music_img': picUrl,
                    'music_id': musicId})

            searchArea.noSingsContentsLabel.hide()
            searchArea.singsResultTable.show()

            self.musicList = musicList

    def itemDoubleClickedEvent(self):
        currentRow = self.searchArea.contentsTab.currentWidget().singsResultTable.currentRow()
        data = self.musicList[currentRow]
        self.searchArea.parent.playWidgets.setPlayerAndPlayList(data)

    def contextEvent(self, event):
        currentWidget = self.searchResultTableIndexs.get(self.currentName)
        if not currentWidget:
            return

        item = currentWidget.itemAt(currentWidget.mapFromGlobal(QCursor.pos()))
        self.menu = QMenu(currentWidget)

        self.menu.addAction(self.actionDownloadSong)
        
        try:
            self.currentIndex = item.row() - 1
        # 在索引是最后一行时会获取不到。
        except:
            self.currentIndex = -1

        self.menu.exec_(QCursor.pos())


class ConfigDetailSings(QObject):
    download = pyqtSignal(dict)

    def __init__(self, parent=None):
        super(ConfigDetailSings, self).__init__()
        self.detailSings = parent
        self.musicList = []

        self.currentIndex = 0

        self.grandparent = self.detailSings.parent
        self.player = self.grandparent.playWidgets.player
        self.playList = self.grandparent.playWidgets
        self.currentMusic = self.grandparent.playWidgets.currentMusic
        self.transTime = transTime

        self.detailSings.singsTable.contextMenuEvent = self.singsFrameContextMenuEvent

        self.bindConnect()
        self.setContextMenu()

    def bindConnect(self):
        self.detailSings.playAllButton.clicked.connect(self.addAllMusicToPlayer)
        self.detailSings.singsTable.itemDoubleClicked.connect(self.itemDoubleClickedEvent)

    def setContextMenu(self):
        self.actionNextPlay = QAction('下一首播放', self)
        self.actionNextPlay.triggered.connect(self.addToNextPlay)

        self.actionDownloadSong = QAction('下载', self)
        self.actionDownloadSong.triggered.connect(self.downloadSong)

    def addToNextPlay(self):
        data = self.musicList[self.currentIndex]
        self.player.setAllMusics([data])
        self.playList.playList.addMusic(data)
        self.playList.playList.addPlayList(data['name'], data['author'], data['time'])

    @toTask
    def downloadSong(self, x):
        musicInfo = self.musicList[self.currentIndex]
        url = musicInfo.get('url')
        if 'http:' not in url and 'https:' not in url:
                songId = musicInfo.get('music_id')
                future = aAsync(netease.singsUrl, [songId])
                url = yield from future
                url = url[0].get('url')
                musicInfo['url'] = url

        self.download.emit(musicInfo)

    def addAllMusicToPlayer(self):
        self.playList.setPlayerAndPlaylists(self.musicList)

    def setupDetailFrames(self, datas, singsUrls, singsIds):
        result = datas
        self.musicList = []
        
        self.detailSings.singsTable.clearContents()

        self.detailSings.titleLabel.setText(result['name'])
        self.detailSings.authorName.setText(result['creator']['nickname'])
        description = result['description']
        # 有些没有简介会报错的。
        if not description:
            description = ''

        self.detailSings.descriptionText.setText(description)
        # 这边添加歌曲的信息到table。
        self.detailSings.singsTable.setRowCount(result['trackCount'])

        for i, j, t, x in zip(result['tracks'], range(result['trackCount']), singsUrls, singsIds):
            names = i['name']
            musicName = QTableWidgetItem(names)
            self.detailSings.singsTable.setItem(j, 0, musicName)

            author = i['artists'][0]['name']
            musicAuthor = QTableWidgetItem(author)
            self.detailSings.singsTable.setItem(j, 1, musicAuthor)

            times = self.transTime(i['duration']/1000)
            musicTime = QTableWidgetItem(times)
            self.detailSings.singsTable.setItem(j, 2, musicTime)

            music_img = i['album']['blurPicUrl']

            lyric = i.get('lyric')

            self.musicList.append({'url': t, 
                                                                'name': names, 
                                                                'time':times, 
                                                                'author':author,
                                                                'music_img': music_img,
                                                                'music_id':x,
                                                                'lyric': lyric})

    # 事件。
    def itemDoubleClickedEvent(self):
        currentRow = self.detailSings.singsTable.currentRow()
        data = self.musicList[currentRow]

        self.playList.setPlayerAndPlayList(data)

    def singsFrameContextMenuEvent(self, event):
        item = self.detailSings.singsTable.itemAt(self.detailSings.singsTable.mapFromGlobal(QCursor.pos()))
        self.menu = QMenu(self.detailSings.singsTable)

        self.menu.addAction(self.actionNextPlay)
        self.menu.addAction(self.actionDownloadSong)
        
        try:
            self.currentIndex = item.row() - 1
        # 在索引是最后一行时会获取不到。
        except:
            self.currentIndex = -1

        self.menu.exec_(QCursor.pos())


class ConfigSystemTray(QObject):

    def __init__(self, systemTray):
        super(ConfigSystemTray, self).__init__()
        self.systemTray = systemTray

        self.addActions()
        self.bindConnect()

    def addActions(self):
        closeAction = QAction('退出', self.systemTray, triggered=self.systemTray.parent.close)

        self.systemTray.addAction(closeAction)
        
    def bindConnect(self):
        pass



"""获取网易云音乐的API。"""
"""暂时没有整改的想法: 一方面旧的API可以用，另一方面逻辑较为简单，当前优先级不高。"""
"""2018/03/18: 相当不RESTful..."""

__author__ = 'cyrbuzz'

import re
import json
import logging
import urllib.parse

from collections import namedtuple

from apiRequestsBase import HttpRequest, ignored
from netEaseEncode import encrypted_request, hashlib


logger = logging.getLogger(__name__)

SongInfo = namedtuple(
    'SongInfo', ['music_id', 'url', 'author', 'time', 'name', 'music_img', 'lyric'])


class NetEaseWebApi(HttpRequest):
    """
        2015年写的，函数名略混乱，不影响使用，暂时不修改。
    """
    cookies = {
        'appver': '2.1.2.184499',
        'os': 'pc',
        'channel': 'netease',
    }

    default_timeout = 10

    def __init__(self):
        super(NetEaseWebApi, self).__init__()
        self.headers['Host'] = 'music.163.com'
        self.headers['Referer'] = 'http://music.163.com'
        self.headers['Content-Type'] = 'application/x-www-form-urlencoded'

        self.vertifyHeaders = self.headers.copy()
        self.vertifyHeaders['Host'] = 'ac.dun.163yun.com'
        self.vertifyHeaders['Accept'] = 'image/png,image/*;q=0.8,*/*;q=0.5'
        self.vertifyHeaders['Content-Type'] = ''

        self.urlEamilHeaders = self.headers.copy()
        self.urlEamilHeaders['Referer'] = ''
        self.urlEamilHeaders['Origin'] = 'orpheus://orpheus'

    def httpRequest(self, *args, **kwargs):
        data = kwargs.get('data')
        if data:
            kwargs['data'] = encrypted_request(data)

        logger.info("进行网易云Url请求, args: {0}, kwargs: {1}".format(args, kwargs))
        html = super(NetEaseWebApi, self).httpRequest(*args, **kwargs)

        with ignored():
            return json.loads(html.text)

        logger.info("url: {0} 请求失败. Header: {1}".format(
            args[0], kwargs.get('headers')))
        return False

    def login(self, username, password):
        """默认记住登陆。"""
        """
            phone 用的/weapi/login/cellphone?csrf_token=
            email 用的/weapi/login?csrf_token=

            还有验证码相关暂时不做。
        """
        password = password.encode()
        md5 = hashlib.md5()
        md5.update(password)
        password = md5.hexdigest()

        data = {'password': password, 'rememberLogin': 'true'}

        # email = data.update({'username': username}) if '@' in username else data.update({'phone': username})
        email = True if '@' in username else False
        if email:
            data['username'] = username
            # https://ac.dun.163yun.com/v1/d?d=nsHeOgdOdpeTDoF4zMLze4xEBfrKIZ2fQzMlHTn2VTgJt%2BkQAVDZ%2B9L5X1DQ%2BkjffAzVQgCLt0EE60l6qm4OGM%2F7FnsP7OO%2B3eZNebXpl6qZt7Q36oeUCUM%2BQcfaX1lsYvHM9vsVpXIJAok6el%2F9uwfX%2BtvF056U%5C8XllLj62g0K22jv2ZYZ8DNU7ws5YSYo%5CRGMRrcJisIyYtP8pbAqiGqi3ZnV%5CLM9568YMdDomJJZc77BTD%5CVwUsj%2Fgv76g%2FCT885OtsSfkOhMVmIESXztxyrLyxt52Vs1wOG%5CyIeR6x%2BaJsYmbjTJXQ2M2DlOv5jqe3PyqHBp0Ar%2FG1ueK5xJve3A0QAS9s4qMXG%2FIpiXvjUT6jA28GNxw3ZFzwNY1IinR%2FH4CooLpNP0bokXAq8Z%2FooRZRZg4uvPa9jHR7l2LlRaTO0oIYvAPvQshOyTq%5CAfUVuLtjyhmkPjgdJNpguxGOKVGqghlYwcrmKuTU8ytT2qxb5x0opJneyEbvGor1LcRC5m9%5CdKlhJ7KbLjdKQGX7l4nHdMe9OS2ViwGXaqHIVKS1Zrck2UsYu1AAjaVOXSOO0Y%2FqhTFHEdrT11vxAf6hRjEdZS8nJxHxv5zsvpx2XXUBUikMum81uF2cdwc0PF1YoITmlGA1IbwXGE3mUfV3Ggvk0%5C1djVd8o3C3F%2BOvQ89V0pq4mBbJ1OSiT4%2FV56qPjLYuIX7leuZGkNYlJIhFypeQDa7CvCh%2BBhVCh%2BO3pt%2FVwbFkrZlmoRM%2BdB22H3z%2F%2Bn%2FWW&cb=_jsonp_ssd1507090923068&x.js
            # https://ac.dun.163yun.com/v1/d?d=FvHNjq2CwM%2FgqdENT6g0AUjuRaNfeeO%5CkX64uXxNGYraS6AVKH0jrEFbxq1y%2BveQPiOClFtY0OqQ8G6OCQuK%2F4zKrVAsaT9XPiPS1%2BhrPj6TT1mX%2BBFFq%2FZafGazTvU13DluLC2Khhid80cbwi95%2BzO4CjubRjNlhJ9VYuoTGErCi8LlSkjUAYplAkfkATJOBQK%2FFg4RAysIEsUpbACTUFqlhQqVL5aBbR8XhN4HooPPOC9L0IY9TFMfI14hhgLc9AiXSq7S5wkjje5fc3tXrfVZqptla8s%2BhFPJNNyKw8IdNJ6Ik9p9aTC8tXkU1yk%2FZeicCMU48zgDvmTjBYVi1xeQ0xxzLZmvA%5C61rJOJNjqC%2Bh0M%2BCy5HtoceoppxfyC4o80YP%5CR3LP5yb03jL89sbqC48mf6a1aTu6d5MnCGnDl17o%2Frk8onMOYLN8YX0Qf3EORGP547CcNlp6ZA83VcwrYGzR%5CYoKbrlLR7dfMb%5CoMusuXm1cGakS6rInXZNQdE%2FFN2OUhlw%5CstCL0UIbw4IsvuwMXl2O5sZhGCejE%2F2%2F1lZ7u57FCHp9BQ4tG5QjJb%2FLQi6V%5CraQLMJxq%5CBuiKbJr7uZrUj4LIq4jsw7jMqmA7o4uM5JMHbzEdg7k%5CsFyM8x9hdeG7owXIDmFuCpfTqiH9wMCZa0DV%2F8m%2BCOuG8q%2F87cGDyvaFlRVoRgnuqQHVE5%2BV5Z6iAMYGtOFQH%2BBLvjPI1cF0yT1twyTR1e0FTnMI3tNjoHH%2Fy%2Fyb1hZCuuIJoJJP%2FWW&cb=_jsonp_ssd1507090962239&x.js
            # a = '1_ZfeoJYpgEKaPdueAuHxAz56XkZ28JiOG_hUOMgOEgFt1H0Azl4sFFeKjDBIbrhKWD'
            data['clientToken'] = "1_ZfeoJYpgEKaPdueAuHxAz56XkZ28JiOG_SrSyeuuSjQrobJdGvXFN2Jo4fzHb+oRQ"
        else:
            data['phone'] = username

        urlEmail = 'http://music.163.com/weapi/login?csrf_token='
        urlPhone = 'http://music.163.com/weapi/login/cellphone?csrf_token='
        url = urlEmail if email else urlPhone

        html = self.httpRequest(url, method='POST', data=data)

        return html

    def user_playlist(self, uid, offset=0, limit=1000):
        """
            2017/8/12更新 新API.
            个人歌单。

        """
        # url = 'http://music.163.com/api/user/playlist/?offset=%s&limit=1000&uid=%s' % (offset, uid)
        # html = self.httpRequest(url, method='GET', cookies=self.cookies)
        # return html['playlist']
        data = {'offset': offset, 'uid': uid, 'limit': limit, 'csrf_token': ''}
        url = 'http://music.163.com/weapi/user/playlist'

        html = self.httpRequest(url, method='POST', data=data)

        try:
            return html['playlist']
        except:
            return {}

    def all_playlist(self, cat='全部歌单', types='all', offset=0, index=1):
        """
            全部歌单。列表字典形式。
        """
        url = 'http://music.163.com/api/playlist/list?cat=%s&type=%s&order=%s&offset=%d&total=true&limit=30&index=%d'\
            % (urllib.parse.quote(cat), types, types, offset, index)

        html = self.httpRequest(url, method='GET', cookies=self.cookies)

        try:
            return html['playlists']
        except:
            return {}

    def details_playlist(self, ids):
        """
            歌单详情。
        """
        url = 'http://music.163.com/api/playlist/detail?id={0}' .format(ids)
        html = self.httpRequest(url, method="GET", cookies=self.cookies)
        with ignored():
            return html['result']
        return False

    def search(self, s, offset=0, limit=100, stype=1):
        """
            搜索.
            type类型: 单曲(1), 专辑(10), 歌手(100), 歌单(1000), 用户(1002)
            2017/7/15更新API.
        """
        # url = 'http://music.163.com/api/search/get/web'
        url = 'http://music.163.com/weapi/cloudsearch/get/web'
        data = {
            's': s,
            'offset': str(offset),
            'limit': str(limit),
            'type': str(stype)
        }

        html = self.httpRequest(url, method='POST', data=data)
        try:
            return html['result']
        except:
            return {'songCount': 0, 'songs': []}

    def singsUrl(self, ids: list):
        """
        2017/7/14更新。
        返回歌曲的URL。
        """

        data = {'csrf_token': '', 'ids': ids, 'br': 999000}
        url = "http://music.163.com/weapi/song/enhance/player/url"
        html = self.httpRequest(url, method='POST', data=data)
        with ignored():
            return html['data']

        logger.info('歌曲请求失败: ids {0}'.format(ids))

        return False

    def newsong(self, areaID=0, offset=0, total='true', limit=100):
        """
            最新音乐--新歌速递。
            areaID(0全部, 9华语, 96欧美, 16韩国, 8日本。)
        """
        url = 'http://music.163.com/api/discovery/new/songs?areaId=%d&offset=%d&total=%s&limit=%d' %\
              (areaID, offset, total, limit)
        html = self.httpRequest(url, method='GET', cookies=self.cookies)

        return html['data']

    def fnewsong(self, year=2015, month=4, area='ALL'):
        """
            最新音乐--新碟上架。
            area(ALL全部, ZH华语, EA欧美, KR韩国, 日本JP)
        """
        url = 'http://music.163.com/api/discovery/new/albums/area?year=%d&month=%d&area=%s&type=hot&offset=0&total=true&limit=20&rcmd=true' \
              % (year, month, area)
        html = self.httpRequest(url, method="GET", cookies=self.cookies)

        return html['monthData']

    def lyric(self, ids):
        url = 'http://music.163.com/api/song/lyric?os=osx&id={0}&lv=-1&kv=-1&tv=-1'.format(
            ids)
        html = self.httpRequest(url, method='GET')

        try:
            return html['lrc']['lyric']
        except:
            return False

    def getContainedPlaylists(self, songId) -> set:
        """

            传入某个歌曲Id, 返回包含此歌曲的3个歌单。
            此功能需要直接解析HTMl文档.
            1. 获取·http://music.163.com/song?id=29953681·
            2. 提取出歌单的id。不想使用额外的包所以直接用正则了。
        """
        rawHtml = super().httpRequest(
            'http://music.163.com/song?id={}'.format(songId), method='GET')

        containedUl = re.findall(
            r'<ul class="m-rctlist f-cb">[.\s\S]+?</ul>', rawHtml.text)
        if not containedUl:
            containedUl = ''
        else:
            containedUl = containedUl[0]

        playlists = set(re.findall(r'data-res-id="(.+)"', containedUl))

        return playlists

    def getRandomSongFromPlaylist(self, playlistId) -> list:
        """
            只返回包含其中音乐信息的列表。
        """
        allSong = self.details_playlist(playlistId)
        if allSong:
            tracks = allSong['tracks']
            SongInfoList = []
            for i in tracks:
                songInfo = SongInfo(music_id=i['id'],
                                    music_img=i['album']['blurPicUrl'],
                                    url="http(0)",
                                    lyric=None,
                                    time=i['duration'],
                                    name=i['name'],
                                    author='-'.join([x['name']
                                                     for x in i['artists']])
                                    )
                SongInfoList.append(songInfo)

            return SongInfoList

        return False


netease = NetEaseWebApi()


if __name__ == '__main__':
    help(netease)



"""推荐歌曲的显示区域"""
__author__ = 'cyrbuzz'

from base import (ScrollArea, QLabel, QFrame, QVBoxLayout, QPushButton, QHBoxLayout, QTableWidget,
                  QAbstractItemView)


class RecommendFrame(ScrollArea):

    def __init__(self, parent=None):
        super().__init__()
        self.parent = parent

        self.setObjectName("recommendFrame")
        with open('QSS/recommendFrames.qss', 'r', encoding='utf-8') as f:
            self.setStyleSheet(f.read())

        self.mainLayout = QVBoxLayout(self)

        self.setHeader()

        self.setMusicTable()

    def setHeader(self):
        self.spaceLine = QFrame(self)
        self.spaceLine.setObjectName("spaceLine")
        self.spaceLine.setFrameShape(QFrame.HLine)
        self.spaceLine.setFrameShadow(QFrame.Plain)
        self.spaceLine.setLineWidth(2)
        
        self.recommendLabel = QLabel("你的专属推荐~❤")
        
        self.topShowLayout = QHBoxLayout()
        self.topShowLayout.addSpacing(20)
        # self.topShowLayout.addWidget(self.titleLabel)
        self.topShowLayout.addWidget(self.recommendLabel)
        self.topShowLayout.addStretch(1)

        self.mainLayout.addLayout(self.topShowLayout)
        self.mainLayout.addWidget(self.spaceLine)

    def setMusicTable(self):
        self.singsTable = QTableWidget()
        self.singsTable.setObjectName('singsTable')
        self.singsTable.setMinimumWidth(self.width())
        self.singsTable.setColumnCount(3)
        self.singsTable.setHorizontalHeaderLabels(['音乐标题', '歌手', '时长'])

        self.singsTable.setColumnWidth(0, self.width()/3*1.25)
        self.singsTable.setColumnWidth(1, self.width()/3*1.25)
        self.singsTable.setColumnWidth(2, self.width()/3*0.5)
        self.singsTable.horizontalHeader().setStretchLastSection(True)
        self.singsTable.verticalHeader().setVisible(False)
        self.singsTable.setShowGrid(False)
        self.singsTable.setAlternatingRowColors(True)

        self.singsTable.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.singsTable.setSelectionBehavior(QAbstractItemView.SelectRows)

        self.mainLayout.addWidget(self.singsTable)


"""主要是网络部分。"""
# 2017/7/26
# 这个文件目前基本无用，预留。
import os
import asyncio

from PyQt5.QtCore import QThread, QObject, QEventLoop, QByteArray, QRunnable, pyqtSignal, pyqtSlot

# 2017/7/25 尝试用协程进行设计。
# 测试发现会卡界面。
# 暂时保留不使用。
# try:
#     from gevent import monkey; monkey.patch_all()
#     from gevent import pool
#     import gevent
#     noGevent = False
# except:
#     noGevent = True
noGevent = True

import httpBase

Requests = httpBase.Requests()
Session = httpBase.Session()


class Pool(object):

    def spawn(self, *args, **kwargs):
        pass

    def join(self):
        pass

# 做成装饰器。
if noGevent:
    pool = Pool()
    def joinJobInGevent(func):

        def join(*args, **kwargs):

            func(*args, **kwargs)

        return join
else:
    pool = pool.Pool()
    def joinJobInGevent(func):

        def join(*args, **kwargs):
            
            pool.spawn(func, *args, **kwargs)

        return join


# class GeventNetWork(object):
    
#     def __new__(cls, *args, **kwargs):
#         """如果没有Gevent库就不要实例化这个类。"""
#         if noGevent:
#             pass
#         else:
#             return object.__new__(cls, *args, **kwargs)
        
#     def __init__(self):
#         pass
    
class ARequests(httpBase.Requests):
    """
    一个异步请求类，
    """
    def __init__(self, callback):
        super().__init__()
        self.callback = callback

    def __enter__(self):
        return self

    def __exit__(self, except_type, value, tb):
        
        return True

    def _httpRequest(self, method, url, kwargs):
        method = method.upper()
        if method == 'GET':
            data = super().get(url, **kwargs)
        elif method == 'POST':
            data = super().post(url, **kwargs)

        return data

    @asyncio.coroutine
    def _get(self, url, **kwargs):
        eventLoop = asyncio.get_event_loop()
        future = eventLoop.run_in_executor(None, self._httpRequest, 'GET', url, kwargs)

        data = yield from future

        return data

    @asyncio.coroutine
    def _post(self, url, **kwargs):
        eventLoop = asyncio.get_event_loop()
        future = eventLoop.run_in_executor(None, self._httpRequest, 'POST', url, kwargs)

        data = yield from future

        return data

    def get(self, url, **kwargs):
        eventLoop = asyncio.get_event_loop()
        future = eventLoop.create_task(self._get(url, **kwargs))
        future.add_done_callback(self.callback)

    def post(self, url, **kwargs):
        eventLoop = asyncio.get_event_loop()
        future = eventLoop.create_task(self._post(url, **kwargs))
        future.add_done_callback(self.callback)


if __name__ == '__main__':
    pass
    import sys
    import time
    import requests
    eventLoop = asyncio.get_event_loop()
    urls = ['https://www.v2ex.com/', 'http://www.baidu.com']*5
    from concurrent.futures import ThreadPoolExecutor

    # b = time.clock()
    with ThreadPoolExecutor(max_workers=5) as f:
        for i in urls:
            f.submit(requests.get, i)
    # def printData(future):
    #     print(future.result())
        
    #     urls.pop()
    #     if not urls:
    #         sys.exit()
    # http = ARequests(printData)
    
    # for i in urls:
    #     http.get(i)
    # eventLoop.run_forever()
